#### HEADER ================================================================
#
# [PURPOSE]
# Example covariate modeling
#
# [AUTHOR]
# Henning Schmidt
#
# [INPUT]
# Data in ../Data/01_dataNLME_PK/dataNLME.csv
#
# [OUTPUT]
# Models in ../Models

## Preliminaries ====

rm(list = ls())
library(IQRtools)
IQRinitCompliance("SCRIPT_20_PK_covariate.R")


# -------------------------------------------------------------------------#
# Define the base model 
# - Linear 2 compartmental distribution model
# - Additive-proportional error model
# - Log-normal IIV distribution for CL, Vc, Q1, Vp1 
# - Diagnonal random effect covariance matrix - FIXED IIV on Q1
# -------------------------------------------------------------------------#

model <- IQRmodel("Resources/model2cpt.txt")

dosing <- dosing_IQRest(
  INPUT1 = c(type="INFUSION")
)

data <- data_IQRest(
  datafile = "../Data/01_dataNLME_PK/dataNLME.csv",
  covNames = c("WT0","AGE0","ALB0","CK0","HT0"),
  catNames = c("SEXF","RACE")
)

modelSpec <- modelSpec_IQRest(
  POPvalues0      = c(CL=0.5,   Vc=3,   Q1=0.5,   Vp1=3),
  IIVdistribution = c(CL="L",   Vc="L", Q1="L",   Vp1="L"),
  IIVestimate     = c(CL=1,     Vc=1,   Q1=2,     Vp1=1),
  IIVvalues0      = c(CL=0.3,   Vc=0.3, Q1=0.1,   Vp1=0.3),
  
  errorModel = list(
    OUTPUT1 = c("absrel",0.2,0.3)
  ) 
)

# -------------------------------------------------------------------------#
# MODELCOV01
# - AGE0 on Vc
# -------------------------------------------------------------------------#

modelSpec$covariateModel <- list(
  Vc = c("AGE0")
)

est <- IQRnlmeEst(model,dosing,data,modelSpec)

# NONMEM SAEM
IQRnlmeProject(est,
               projectPath = "../Models/03_covariate_modeling/MODELCOV01",
               comment="AGE0 on Vc",
               tool = "NONMEM",
               algOpt.NONMEM.METHOD = "SAEM")

run_IQRnlmeProject("../Models/03_covariate_modeling/MODELCOV01",ncores = 4)


# -------------------------------------------------------------------------#
# MODELCOV02
# - AGE0 on Vc
# - SEXF on CL and Vc
# -------------------------------------------------------------------------#

modelSpec$covariateModel <- list(
  Vc = c("AGE0","SEXF"),
  CL = c("SEXF")
)

est <- IQRnlmeEst(model,dosing,data,modelSpec)

# NONMEM SAEM
IQRnlmeProject(est,
               projectPath = "../Models/03_covariate_modeling/MODELCOV02",
               comment="AGE0 on Vc, SEX on CL and Vc",
               tool = "NONMEM",
               algOpt.NONMEM.METHOD = "SAEM")

run_IQRnlmeProject("../Models/03_covariate_modeling/MODELCOV02",ncores = 4)

# -------------------------------------------------------------------------#
# MODELCOV03
# - AGE0 on Vc and CL
# - SEXF on CL and Vc
# - ALB0 on CL and Vc
# - CK0 on CL
# -------------------------------------------------------------------------#

modelSpec$covariateModel <- list(
  Vc = c("AGE0","SEXF","ALB0"),
  CL = c("SEXF","ALB0","CK0","AGE0","WT0")
)

est <- IQRnlmeEst(model,dosing,data,modelSpec)

# NONMEM SAEM
IQRnlmeProject(est,
               projectPath = "../Models/03_covariate_modeling/MODELCOV03",
               comment="AGE0 on CL and Vc, SEX on CL and Vc, ALB0 on CL and Vc",
               tool = "NONMEM",
               algOpt.NONMEM.METHOD = "SAEM")

run_IQRnlmeProject("../Models/03_covariate_modeling/MODELCOV03",ncores = 4)



# -------------------------------------------------------------------------#
# Generate a summary
# -------------------------------------------------------------------------#

pM <- as_IQRnlmeProjectMulti("../Models/03_covariate_modeling/")
summary(pM,pathname = "../Models/03_covariate_modeling/",FLAGremovePath = TRUE)

# -------------------------------------------------------------------------#
# Comparison table
# -------------------------------------------------------------------------#

pM <- as_IQRnlmeProjectMulti("../Models/03_covariate_modeling/")
names(pM) <- gsub("../Models/03_covariate_modeling//","",names(pM))
compareModels_IQRnlmeProjectMulti(pM,filename = "../Models/03_covariate_modeling/00_Comparison.txt")

# -------------------------------------------------------------------------#
# Select final covariate model
# -------------------------------------------------------------------------#

duplicate_IQRnlmeProject(projectSource = "../Models/03_covariate_modeling/MODELCOV01",
                         projectDestination = "../Output/FINALMODELS/02_COVARIATEMODEL",
                         FLAGcopyData = TRUE)
