#### HEADER ================================================================
#
# [PURPOSE]
# Process source data into Raw General Dataset Format
#
# [AUTHOR]
# Henning Schmidt
#
# [INPUT]
# Data files in ../DataSource/
#
# [OUTPUT]
# Processed raw GDF data in ../Output/01_importData

## Preliminaries ====

rm(list = ls())
library(IQRtools)
IQRinitCompliance("SCRIPT_01_importData.R")

# -------------------------------------------------------------------------#
# Import of source data
# -------------------------------------------------------------------------#

dm  <- IQRloadSASdata('../DataSource/dm.xpt') # Demographics
vs  <- IQRloadSASdata('../DataSource/vs.xpt') # Vital signs
pc  <- IQRloadSASdata('../DataSource/pc.xpt') # PK
lb  <- IQRloadSASdata('../DataSource/lb.xpt') # Lab values
ex  <- IQRloadSASdata('../DataSource/ex.xpt') # Dose administration

# -------------------------------------------------------------------------#
# Data format for parsing information in:
# USUBJID:  Unique Subject ID
# STDTC:    Date and time of event start
# ENDTC:    Date and time of event end (NA is duration 0)
# TYPENAME: "Demographics", "Dose", "PK"
# NAME:     Name of the assessemnt
# VALUE:    Value of the assessment
# UNIT:     Unit of the assessment
# ROUTE:    IV for IV dosing, SUBCUT for SC dosing, NA for non-dose records
# LLOQ:     LLOQ of measurement
# SCREEN:   Screening flag
# BASE:     Baseline flag
# -------------------------------------------------------------------------#

# -------------------------------------------------------------------------#
# Demographics
# -------------------------------------------------------------------------#

# Extract relevant data
data_dm <- data.frame(
  USUBJID    = dm$USUBJID,
  STDTC   = dm$RFXSTDTC,
  ENDTC   = NA,
  TYPENAME   = "Demographics",
  UNIT       = NA,
  ROUTE      = NA,
  LLOQ       = NA,
  SCREEN     = 1,
  BASE       = 0,
  Age        = dm$AGE,
  Gender     = dm$SEX,
  Race       = dm$RACE,
  Ethnicity  = dm$ETHNIC,
  stringsAsFactors = FALSE
)
data_dm <- suppressWarnings(tidyr::gather(data_dm,NAME,VALUE,Age,Gender,Race,Ethnicity))
data_dm$UNIT[data_dm$NAME=="Age"] <- tolower(unique(dm$AGEU)[1])


# -------------------------------------------------------------------------#
# Vital signs (Height and Weight)
# Keep only baseline values
# -------------------------------------------------------------------------#

data_vs <- data.frame(
  USUBJID    = vs$USUBJID,
  STDTC      = vs$VSDTC,
  ENDTC      = NA,
  TYPENAME   = "Vital Signs",
  NAME       = vs$VSTEST,
  VALUE      = vs$VSORRES,
  UNIT       = vs$VSORRESU,
  ROUTE      = NA,
  LLOQ       = NA,
  SCREEN     = 0,
  BASE       = as.numeric(vs$VSBLFL=="Y"),
  stringsAsFactors = FALSE
)
data_vs <- data_vs[data_vs$NAME %in% c("Height","Weight") & data_vs$BASE==1,]


# -------------------------------------------------------------------------#
# PK data
# -------------------------------------------------------------------------#

data_pc <- data.frame(
  USUBJID    = pc$USUBJID,
  STDTC      = pc$PCDTC,
  ENDTC      = NA,
  TYPENAME   = "PK",
  NAME       = pc$PCTEST,
  VALUE      = pc$PCORRES,
  UNIT       = pc$PCORRESU,
  ROUTE      = NA,
  LLOQ       = pc$PCLLOQ,
  SCREEN     = 0,
  BASE       = 0,
  stringsAsFactors = FALSE
)


# -------------------------------------------------------------------------#
# Lab values (Albumin, Bilirubin, Creatine Kinase")
# Keep only baseline values
# -------------------------------------------------------------------------#

data_lb <- data.frame(
  USUBJID    = lb$USUBJID,
  STDTC   = lb$LBDTC,
  ENDTC   = NA,
  TYPENAME   = "Lab Values",
  NAME       = lb$LBTEST,
  VALUE      = lb$LBORRES,
  UNIT       = lb$LBORRESU,
  ROUTE      = NA,
  LLOQ       = NA,
  SCREEN     = 0,
  BASE       = as.numeric(lb$LBBLFL=="Y"),
  stringsAsFactors = FALSE
)
data_lb <- data_lb[data_lb$NAME %in% c("Albumin","Creatine Kinase") & data_lb$BASE==1,]


# -------------------------------------------------------------------------#
# Dosing records
# -------------------------------------------------------------------------#

data_ex <- data.frame(
  USUBJID    = ex$USUBJID,
  STDTC      = ex$EXSTDTC,
  ENDTC      = ex$EXENDTC,
  TYPENAME   = "Dose",
  NAME       = "HS0815 Dose",
  VALUE      = ex$EXDOSE,
  UNIT       = ex$EXDOSU,
  ROUTE      = "IV",
  LLOQ       = NA,
  SCREEN     = 0,
  BASE       = 0,
  stringsAsFactors = FALSE
)


# -------------------------------------------------------------------------#
# Sort columns and combine panels
# -------------------------------------------------------------------------#

order_columns <- c("USUBJID","STDTC","ENDTC","TYPENAME","NAME","VALUE","UNIT","ROUTE","LLOQ","SCREEN","BASE")
panels <- list(data_dm,data_vs,data_pc,data_lb,data_ex)
data <- do.call(rbind,lapply(panels, function (p) p[,order_columns]))


# -------------------------------------------------------------------------#
# Calculate a TIME column
# -------------------------------------------------------------------------#

data <- dplyr::arrange(data,USUBJID,STDTC)
sd <- split(data,data$USUBJID)
data <- do.call(rbind,lapply(seq_along(sd), function (k) {
  d <- sd[[k]]
  starttime <- date2time_IQRdataProgramming(d$STDTC)$time_days
  endtime <- date2time_IQRdataProgramming(d$ENDTC)$time_days
  duration <- endtime-starttime
  duration[is.na(duration)] <- 0
  d$TIME <- starttime-min(starttime[d$TYPENAME=="Dose"],na.rm = TRUE)
  d$TIMEUNIT <- "Days"
  d$DURATION <- duration
  d
}))


# -------------------------------------------------------------------------#
# Splitting VALUE and VALUETXT
# -------------------------------------------------------------------------#
value <- suppressWarnings(as.numeric(data$VALUE))
data$VALUETXT <- NA
data$VALUETXT[is.na(value)] <- data$VALUE[is.na(value)]
data$VALUE <- value
data$VALUE[is.na(value)] <- NA


# -------------------------------------------------------------------------#
# Define numeric versions of VALUETXT
# -------------------------------------------------------------------------#

# Race
data$VALUE[data$NAME=="Race" & data$VALUETXT=="WHITE"] <- 1
data$VALUE[data$NAME=="Race" & data$VALUETXT=="BLACK OR AFRICAN AMERICAN"] <- 2
data$VALUE[data$NAME=="Race" & data$VALUETXT=="ASIAN"] <- 3
data$VALUE[data$NAME=="Race" & data$VALUETXT=="AMERICAN INDIAN OR ALASKA NATIVE"] <- 4
data$VALUE[data$NAME=="Race" & data$VALUETXT=="NATIVE HAWAIIAN OR OTHER PACIFIC ISLANDER"] <- 5
data$VALUE[data$NAME=="Race" & data$VALUETXT=="OTHER"] <- 6

# Ethnicity
data$VALUE[data$NAME=="Ethnicity" & data$VALUETXT=="HISPANIC OR LATINO"] <- 1
data$VALUE[data$NAME=="Ethnicity" & data$VALUETXT=="NOT HISPANIC OR LATINO"] <- 2
data$VALUE[data$NAME=="Ethnicity" & data$VALUETXT=="NOT REPORTED"] <- 3
data$VALUE[data$NAME=="Ethnicity" & data$VALUETXT=="UNKNOWN"] <- 4

# Gender
data$VALUE[data$NAME=="Gender" & data$VALUETXT=="M"] <- 0
data$VALUE[data$NAME=="Gender" & data$VALUETXT=="F"] <- 1


# -------------------------------------------------------------------------#
# Add final information
# -------------------------------------------------------------------------#

data$STUDY <- dm$STUDYID[1]
data$COMPOUND <- pc$PCTESTCD[1]

# -------------------------------------------------------------------------#
# Done! Export processed raw data
# -------------------------------------------------------------------------#

dataOut <- data[,c("COMPOUND","STUDY","USUBJID","STDTC","ENDTC","TIME","TIMEUNIT","DURATION","SCREEN","BASE",
                   "TYPENAME","NAME","VALUE","VALUETXT","UNIT","ROUTE","LLOQ")]

IQRoutputCSV(dataOut,"../Output/01_importData/DAT01_GDF_raw.csv")
