#### HEADER ================================================================
#
# [PURPOSE]
# Example covariance modeling
#
# [AUTHOR]
# Henning Schmidt
#
# [INPUT]
# Data in ../Data/01_dataNLME_PK/dataNLME.csv
#
# [OUTPUT]
# Models in ../Models

## Preliminaries ====

rm(list = ls())
library(IQRtools)
IQRinitCompliance("SCRIPT_25_PK_covariance.R")


# -------------------------------------------------------------------------#
# Define the covariate model 
# - Linear 2 compartmental distribution model
# - Additive-proportional error model
# - Log-normal IIV distribution for CL, Vc, Q1, Vp1 
# - Diagnonal random effect covariance matrix - FIXED IIV on Q1
# - AGE0 on Vc
# -------------------------------------------------------------------------#

model <- IQRmodel("Resources/model2cpt.txt")

dosing <- dosing_IQRest(
  INPUT1 = c(type="INFUSION")
)

data <- data_IQRest(
  datafile = "../Data/01_dataNLME_PK/dataNLME.csv",
  covNames = c("WT0","AGE0","ALB0","CK0","HT0"),
  catNames = c("SEXF","RACE")
)

modelSpec <- modelSpec_IQRest(
  POPvalues0      = c(CL=0.5,   Vc=3,   Q1=0.5,   Vp1=3),
  IIVdistribution = c(CL="L",   Vc="L", Q1="L",   Vp1="L"),
  IIVestimate     = c(CL=1,     Vc=1,   Q1=2,     Vp1=1),
  IIVvalues0      = c(CL=0.3,   Vc=0.3, Q1=0.1,   Vp1=0.3),
  
  errorModel = list(
    OUTPUT1 = c("absrel",0.2,0.3)
  ),
  
  covariateModel = list(
    Vc = c("AGE0")
  )
)

# -------------------------------------------------------------------------#
# MODELCOR01
# - CL/Vc
# -------------------------------------------------------------------------#

modelSpec$covarianceModel = c("CL,Vc")

est <- IQRnlmeEst(model,dosing,data,modelSpec)

# NONMEM SAEM
IQRnlmeProject(est,
               projectPath = "../Models/04_covariance_modeling/MODELCOR01",
               comment="CL/Vc",
               tool = "NONMEM",
               algOpt.NONMEM.METHOD = "SAEM")

run_IQRnlmeProject("../Models/04_covariance_modeling/MODELCOR01",ncores = 4)


# -------------------------------------------------------------------------#
# MODELCOR02
# - CL/Vc/Vp1
# -------------------------------------------------------------------------#

modelSpec$covarianceModel = c("CL,Vc,Vp1")

est <- IQRnlmeEst(model,dosing,data,modelSpec)

# NONMEM SAEM
IQRnlmeProject(est,
               projectPath = "../Models/04_covariance_modeling/MODELCOR02",
               comment="CL/Vc/Vp1",
               tool = "NONMEM",
               algOpt.NONMEM.METHOD = "SAEM")

run_IQRnlmeProject("../Models/04_covariance_modeling/MODELCOR02",ncores = 4)

# -------------------------------------------------------------------------#
# Generate a summary
# -------------------------------------------------------------------------#

pM <- as_IQRnlmeProjectMulti("../Models/04_covariance_modeling/")
summary(pM,pathname = "../Models/04_covariance_modeling/",FLAGremovePath = TRUE)

# -------------------------------------------------------------------------#
# Comparison table
# -------------------------------------------------------------------------#

pM <- as_IQRnlmeProjectMulti("../Models/04_covariance_modeling/")
names(pM) <- gsub("../Models/04_covariance_modeling//","",names(pM))
compareModels_IQRnlmeProjectMulti(pM,filename = "../Models/04_covariance_modeling/00_Comparison.txt")

# -------------------------------------------------------------------------#
# Select final covariate model
# -------------------------------------------------------------------------#

duplicate_IQRnlmeProject(projectSource = "../Models/04_covariance_modeling/MODELCOR01",
                         projectDestination = "../Output/FINALMODELS/03_FINALMODEL",
                         FLAGcopyData = TRUE)
